<?php

namespace Modules\Pp\Http\Controllers;

use Exception;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Modules\Hr\Entities\AttShift;
use Modules\Hr\Entities\EmployeeSite;
use Modules\Pd\Entities\ScheduleMachineryPlan;
use Modules\Pd\Entities\ScheduleOrderResourceBom;
use Modules\Pd\Entities\ScheduleOrderResourceHr;
use Modules\Pp\Entities\CustomerOrder;
use Modules\Pp\Entities\CustomerOrderData;
use Modules\Pp\Entities\CustomerPlanData;
use Modules\Pp\Entities\Machinary;
use Modules\Pp\Entities\MachineryPlan;
use Modules\St\Entities\Product;

class ProductionPlanListController extends Controller
{
    public function productionPlan()
    {
        try {
            $query = 'SELECT pp_customer_order_plans.trans_date,
            pp_customer_orders.purchase_order,
            pp_customer_orders.factory_po_num,
            st_products.product_id,
            st_products.product_name,
            MIN(pp_machinery_plans.date) AS from_date,
            MAX(pp_machinery_plans.date) AS to_date,
            hr_employee_sites.site AS factory,
            pp_machinery_plans.machinery_id,
            pp_machinery_plans.product_primary_id,
            pp_machinery_plans.customer_order_plan_data_id
            FROM pp_customer_order_plans
            INNER JOIN pp_customer_orders ON pp_customer_order_plans.order_id = pp_customer_orders.order_id
            INNER JOIN pp_customer_order_plans_data ON pp_customer_order_plans.customer_order_plan_id = pp_customer_order_plans_data.customer_order_plan_id
            INNER JOIN st_products ON pp_customer_order_plans_data.product_id = st_products.product_primary_id
            INNER JOIN pp_machinery_plans ON pp_customer_order_plans_data.customer_order_plan_data_id = pp_machinery_plans.customer_order_plan_data_id
            INNER JOIN pp_machinery ON pp_machinery_plans.machinery_id = pp_machinery.machinery_id
            INNER JOIN hr_employee_sites ON pp_machinery.site_id = hr_employee_sites.site_id GROUP BY pp_machinery_plans.customer_order_plan_data_id';
            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "ProductionScheduleCreateController", "productionPlan", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "ProductionScheduleCreateController", "error", $exception->getMessage());
        }

        return response()->json(["data" => $responseBody]);
    }










    function getMachineryShifts($id)
    {
        try {
            $query = 'SELECT DISTINCT hr_att_shifts.shift_id,hr_att_shifts.shift_name FROM hr_att_shifts 
            INNER JOIN pp_machinery_plans ON hr_att_shifts.shift_id = pp_machinery_plans.shift_id
            WHERE pp_machinery_plans.customer_order_plan_data_id = "' . $id . '"';
            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "getMachineryShifts", "Found", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "getMachineryShifts", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }



    function getSavedAllocatedDataToDetailsHR($customer_order_plan_data_id, $fg_product_id, $shift_id)
    {
        try {
            $query = 'SELECT hr_employee_designations.designation_id,hr_employee_designations.designation,pp_order_resource_hrs.no_of_heads AS count FROM hr_employee_designations INNER JOIN pp_production_teams ON hr_employee_designations.designation_id = pp_production_teams.designation_id
            INNER JOIN pp_order_resource_hrs ON pp_production_teams.designation_id = pp_order_resource_hrs.designation_id WHERE pp_order_resource_hrs.customer_order_plan_data_id = "' . $customer_order_plan_data_id . '" AND pp_order_resource_hrs.fg_product_id = "' . $fg_product_id . '" AND pp_order_resource_hrs.shift_id = "' . $shift_id . '"';
            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "getSavedAllocatedDataToDetailsHR", "Found", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "getSavedAllocatedDataToDetailsHR", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }




    public function getSelectedProductData($customer_order_plan_data_id, $product_primary_id, $machine_id)
    {
        try {

            $allocated_qty = $this->getAllocatedQuantity($customer_order_plan_data_id, $product_primary_id);
            $bom_header_qty = $this->getBomHeaderQuantityProduct($product_primary_id);
            $customerPlanData =  CustomerPlanData::find($customer_order_plan_data_id);
            if ($customerPlanData) {
                $customerPlanData->factory_po = "";
                $customerPlanData->customer_po = "";
                $customerPlanData->product_code = "";
                $customerPlanData->machine = "";
                $customerPlanData->factory = "";
                $customerPlanData->allocated_quantity = $allocated_qty;
                $customerPlanData->bom_header_quantity = $bom_header_qty;
                $orderData = CustomerOrderData::find($customerPlanData->order_data_id);
                if ($orderData) {
                    $customerOrder = CustomerOrder::find($orderData->order_id);
                    if ($customerOrder) {
                        $customerPlanData->factory_po = $customerOrder->factory_po_num;
                        $customerPlanData->customer_po = $customerOrder->purchase_order;
                    }
                }
                $product = Product::find($customerPlanData->product_id);
                if ($product) {
                    $customerPlanData->product_code = $product->product_id;
                }

                $machine = Machinary::find($machine_id);
                if ($machine) {
                    $customerPlanData->machine = $machine->machinery_no . "-" . $machine->machinery_name;
                    $site = EmployeeSite::find($machine->site_id);
                    if ($site) {
                        $customerPlanData->factory = $site->site;
                    }
                }
            }
            $responseBody = $this->responseBody(true, "ProductionScheduleCreateController", "getSelectedProductData", $customerPlanData);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "ProductionScheduleCreateController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    function getAllocatedQuantity($order_id, $product_id)
    {

        $quantity = 0;
        $query = 'SELECT SUM(pp_machinery_plans.quantity) AS quantity FROM pp_machinery_plans 
            WHERE pp_machinery_plans.customer_order_plan_data_id = "' . $order_id . '" AND pp_machinery_plans.product_primary_id = "' . $product_id . '"';
        $result = DB::select($query);
        foreach ($result as $res) {
            $quantity = $res->quantity;
        }
        if (!$quantity) {
            $quantity = 0;
        }
        return $quantity;
    }


    function getBomHeaderQuantityProduct($product_id)
    {

        $quantity = 0;
        $query = 'SELECT pp_bom_header.quantity FROM pp_bom_header WHERE pp_bom_header.product_id = "' . $product_id . '"';
        $result = DB::select($query);
        foreach ($result as $res) {
            $quantity = $res->quantity;
        }
        if (!$quantity) {
            $quantity = 0;
        }
        return $quantity;
    }




    
    function getSavedAllocatedBOMData($customer_order_plan_data_id, $fg_product_id)
    {
        try {
            $query = 'SELECT st_product_units.name AS uom_name,st_products.product_primary_id,st_products.product_id,st_products.product_name,SUM(sc_product_histories.quantity) AS stock_in_h,pp_order_resource_boms.* FROM st_products
            INNER JOIN pp_order_resource_boms ON st_products.product_primary_id = pp_order_resource_boms.rm_product_id
            INNER JOIN st_product_units ON pp_order_resource_boms.uom = st_product_units.id
			LEFT JOIN sc_product_histories ON pp_order_resource_boms.fg_product_id = sc_product_histories.product_id
            WHERE  pp_order_resource_boms.customer_order_plan_data_id = "' . $customer_order_plan_data_id . '" AND pp_order_resource_boms.fg_product_id = "' . $fg_product_id . '"';
            $result = DB::select($query);
            $responseBody = $this->responseBody(true, "getSavedAllocatedBOMData", "Found", $result);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "getSavedAllocatedBOMData", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }


    function getSavedMachineryData($order_id, $product_id, $machinery_id)
    {
        try {
            $machineProducts =  MachineryPlan::where([['customer_order_plan_data_id', '=', $order_id], ['product_primary_id', '=', $product_id], ['machinery_id', '=', $machinery_id]])->get();
            foreach ($machineProducts as $mp) {
                $mp->shift_name = "";
                $shift = AttShift::find($mp->shift_id);
                if ($shift) {
                    $mp->shift_name = $shift->shift_name;
                }
            }
            $responseBody = $this->responseBody(true, "getAllocatedData", "Found", $machineProducts);
        } catch (\Exception $ex) {
            $responseBody = $this->responseBody(false, "getAllocatedData", "error", $ex);
        }
        return response()->json(["data" => $responseBody]);
    }
    




    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
